﻿//=======================================================================================
//
//  Purpose: Implement the ViewModel for the NE555Calulator.
//
//
//  Copyright (C) 2011 Mark Stevens
//
//  Permission is hereby granted, free of charge, to any person obtaining a copy
//  of this software and associated documentation files (the "Software") to use
//  the Software in a non-commercial environment subject to the following conditions:
//
//  1.  The above copyright notice and this permission notice shall be included in
//      all copies or substantial portions of the Software.
//  2.  The original author shall be acknowledged in any software and documentation
//      including (but not restricted to) books, articles and online posts.
//
//  Use in a commercial environment is subject to the written consent of the author
//  and/or copyright holder.
//
//  The Software, documentation, instructions for use and associated materials
//  are provided "as is" without warranty of any kind express or implied.  The
//  author and copyright holders shall ihn no way be held liable for any claim
//  or damages arising as a result of the use of the software and/or documentation.
//
//=======================================================================================
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Net;
using System.Net.Sockets;
using System.Windows;

namespace NE555Calculator
{
    /// <summary>
    /// Model for the NE555 timer chip.
    /// </summary>
    public class NE555CalculatorViewModel : INotifyPropertyChanged
    {
        #region Constants

        /// <summary>
        /// DDL item text for the Mode - Calculate the frequency
        /// </summary>
        private const string DI_CALCULATE_FREQUENCY = "Calculate Frequency";

        /// <summary>
        /// DDL item text for the Mode - Calculate the value for resistor 1
        /// </summary>
        private const string DI_CALCULATE_R1 = "Calculate R1";

        /// <summary>
        /// DDL item text for the Mode - Calculate the value for resistor 2
        /// </summary>
        private const string DI_CALCULATE_R2 = "Calculate R2";

        /// <summary>
        /// DDL item text for the Mode - Calculate the value for the capacitor
        /// </summary>
        private const string DI_CALCULATE_C1 = "Calculate C1";

        /// <summary>
        /// DDL items text component type - Standard value
        /// </summary>
        private const string DI_STANDARD_VALUE = "Standard value";

        /// <summary>
        /// DDL items text component type - User specified value
        /// </summary>
        private const string DI_SPECIFIC_VALUE = "Specific value";

        /// <summary>
        /// DDL items text component type - Range of component values
        /// </summary>
        private const string DI_RANGE_OF_VALUES = "Range of values";

        /// <summary>
        /// Text for the connection button when we are disconnected.
        /// </summary>
        private const string BT_CONNECT = "Connect";

        /// <summary>
        /// Text for the connection button when we are connected.
        /// </summary>
        private const string BT_DISCONNECT = "Disconnect";

        #endregion

        #region Enums

        /// <summary>
        /// Enum indicating the current netqork operation being performed.
        /// </summary>
        private enum SocketOperations
        {
            None,
            Connecting,
            Listening,
            Sending,
            Receiving
        }

        #endregion

        #region Properties

        /// <summary>
        /// Collection of standard resistors.
        /// </summary>
        private ObservableCollection<Component> _standardResistors;
        public ObservableCollection<Component> StandardResistors
        {
            get { return (_standardResistors); }
            set
            {
                _standardResistors = value;
                RaisePropertyChanged("StandardResistors");
            }
        }

        /// <summary>
        /// Collection of standard capacitors.
        /// </summary>
        private ObservableCollection<Component> _standardCapacitors;
        public ObservableCollection<Component> StandardCapacitors
        {
            get { return (_standardCapacitors); }
            set
            {
                _standardCapacitors = value;
                RaisePropertyChanged("StandardCapacitors");
            }
        }

        /// <summary>
        /// Calculation mode for the application.
        /// </summary>
        private ObservableCollection<string> _mode;
        public ObservableCollection<string> Mode
        {
            get { return (_mode); }
            set
            {
                _mode = value;
                RaisePropertyChanged("Mode");
            }
        }

        /// <summary>
        /// Selected calculation mode.
        /// </summary>
        private string _selectedMode;
        public string SelectedMode
        {
            get { return (_selectedMode); }
            set
            {
                _selectedMode = value;
                RaisePropertyChanged("SelectedMode");
            }
        }

        /// <summary>
        /// Component value types for the DDL.
        /// </summary>
        public ObservableCollection<string> ComponentValueType { get; set; }

        /// <summary>
        /// Resistor 1 component value type.
        /// </summary>
        private string _r1ComponentValueType;
        public string R1ComponentValueType
        {
            get { return (_r1ComponentValueType); }
            set
            {
                _r1ComponentValueType = value;
                RaisePropertyChanged("R1ComponentValueType");
            }
        }

        /// <summary>
        /// Restistor R1 standard component selected by the user.
        /// </summary>
        public Component R1SelectedStandardValue { get; set; }

        /// <summary>
        /// Restistor R1 standard component starting value selected by the user.
        /// </summary>
        public Component R1SelectedStartValue { get; set; }

        /// <summary>
        /// Restistor R1 standard component final value selected by the user.
        /// </summary>
        public Component R1SelectedFinalValue { get; set; }

        /// <summary>
        /// User selected value for R2.
        /// </summary>
        public double R1UserValue { get; set; }

        /// <summary>
        /// Resistor 2 component value type.
        /// </summary>
        private string _r2ComponentValueType;
        public string R2ComponentValueType
        {
            get { return (_r2ComponentValueType); }
            set
            {
                _r2ComponentValueType = value;
                RaisePropertyChanged("R2ComponentValueType");
            }
        }

        /// <summary>
        /// Restistor R2 standard component selected by the user.
        /// </summary>
        public Component R2SelectedStandardValue { get; set; }

        /// <summary>
        /// Restistor R2 standard component starting value selected by the user.
        /// </summary>
        public Component R2SelectedStartValue { get; set; }

        /// <summary>
        /// Restistor R2 standard component final value selected by the user.
        /// </summary>
        public Component R2SelectedFinalValue { get; set; }

        /// <summary>
        /// User selected value for the R2.
        /// </summary>
        public double R2UserValue { get; set; }

        /// <summary>
        /// Capacitor C1 standard component value selected by the user.
        /// </summary>
        public Component C1SelectedStandardValue { get; set; }

        /// <summary>
        /// Capacitor 1 component value type.
        /// </summary>
        private string _c1ComponentValueType;
        public string C1ComponentValueType
        {
            get { return (_c1ComponentValueType); }
            set
            {
                _c1ComponentValueType = value;
                RaisePropertyChanged("C1ComponentValueType");
            }
        }

        /// <summary>
        /// Restistor R1 standard component starting value selected by the user.
        /// </summary>
        public Component C1SelectedStartValue { get; set; }

        /// <summary>
        /// Restistor R1 standard component final value selected by the user.
        /// </summary>
        public Component C1SelectedFinalValue { get; set; }

        /// <summary>
        /// User entered value to use for the capacitor.
        /// </summary>
        public double C1UserValue { get; set; }

        /// <summary>
        /// User entered frequency.
        /// </summary>
        public double FrequencyValue { get; set; }

        /// <summary>
        /// Should the results panel be shown?
        /// </summary>
        public Visibility ShowSingleCalcuationResults { get; set; }

        /// <summary>
        /// Show the results in a DataGrid?
        /// </summary>
        /// <remarks>
        /// This is used when the user selects to use a range of values.
        /// </remarks>
        public Visibility ShowDataGridResults { get; set; }

        /// <summary>
        /// Collection of results for display in the results DataGrid.
        /// </summary>
        private ObservableCollection<ResultItem> _results;
        public ObservableCollection<ResultItem> Results
        {
            get { return (_results); }
            set
            {
                _results = value;
                RaisePropertyChanged("Results");
            }
        }

        /// <summary>
        /// Hold a one shot result of the calcuation.
        /// </summary>
        public ResultItem SingleResult { get; set; }

        /// <summary>
        /// Test for the connection button.
        /// </summary>
        private string _connectionButtonText;
        public string ConnectionButtonText
        {
            get { return (_connectionButtonText); }
            set
            {
                _connectionButtonText = value;
                RaisePropertyChanged("ConnectionButtonText");
            }
        }

        /// <summary>
        /// IP Address of the Netduino Plus.
        /// </summary>
        public string NetduinoIPAddress { get; set; }

        /// <summary>
        /// Port number of the Netdunio Plus.
        /// </summary>
        public int NetduinoPortNumber { get; set; }

        /// <summary>
        /// Name of the file on the Netduino's SD card.
        /// </summary>
        public string FileName { get; set; }

        /// <summary>
        /// Sample rate for teh Netduino
        /// </summary>
        private int _sampleRate;
        public int SampleRate 
        {
            get { return (_sampleRate); }
            set
            {
                _sampleRate = value;
            }
        }

        /// <summary>
        /// Socket used for the connection to the Netduino Plus.
        /// </summary>
        private Socket Connection { get; set; }

        /// <summary>
        /// End point for the connection to the Netduino Plus.
        /// </summary>
        private DnsEndPoint Endpoint { get; set; }

        /// <summary>
        /// Status text for the main page.
        /// </summary>
        private string _status;
        public string Status
        {
            get { return (_status); }
            set
            {
                _status = value;
                RaisePropertyChanged("Status");
            }
        }

        /// <summary>
        /// Readings from the Netduino.
        /// </summary>
        private ObservableCollection<DataItem> _readings;
        public ObservableCollection<DataItem> Readings
        {
            get { return (_readings); }
            set
            {
                _readings = value;
                RaisePropertyChanged("Readings");
            }
        }

        /// <summary>
        /// Current socket operation being performed.
        /// </summary>
        private SocketOperations CurrentOpertion { get; set; }

        /// <summary>
        /// Is data available and should the data plot be visible?
        /// </summary>
        private Visibility _chartVisibility;
        public Visibility ChartVisibility
        {
            get { return (_chartVisibility); }
            set
            {
                _chartVisibility = value;
                RaisePropertyChanged("ChartVisibility");
            }
        }

        #endregion

        #region Constructor(s)

        /// <summary>
        /// Default constructor
        /// </summary>
        public NE555CalculatorViewModel()
        {
            #region Setup the calculation mode DDL.

            Mode = new ObservableCollection<string>();
            Mode.Add(DI_CALCULATE_FREQUENCY);
            Mode.Add(DI_CALCULATE_R1);
            Mode.Add(DI_CALCULATE_R2);
            Mode.Add(DI_CALCULATE_C1);
            SelectedMode = DI_CALCULATE_FREQUENCY;

            #endregion

            #region Setup the component type DDL values.

            ComponentValueType = new ObservableCollection<string>();
            ComponentValueType.Add(DI_STANDARD_VALUE);
            ComponentValueType.Add(DI_RANGE_OF_VALUES);
            ComponentValueType.Add(DI_SPECIFIC_VALUE);
            R1ComponentValueType = DI_STANDARD_VALUE;
            R2ComponentValueType = DI_STANDARD_VALUE;
            C1ComponentValueType = DI_STANDARD_VALUE;

            #endregion

            #region Add Standard Resisitors to the list.

            StandardResistors = new ObservableCollection<Component>();
            StandardResistors.Add(new Component("1 ohm", 1));
            StandardResistors.Add(new Component("1.1 ohm", 1.1));
            StandardResistors.Add(new Component("1.2 ohm", 1.2));
            StandardResistors.Add(new Component("1.3 ohm", 1.3));
            StandardResistors.Add(new Component("1.5 ohm", 1.5));
            StandardResistors.Add(new Component("1.6 ohm", 1.6));
            StandardResistors.Add(new Component("1.8 ohm", 1.8));
            StandardResistors.Add(new Component("2.0 ohm", 2.0));
            StandardResistors.Add(new Component("2.2 ohm", 2.2));
            StandardResistors.Add(new Component("2.4 ohm", 2.4));
            StandardResistors.Add(new Component("2.7 ohm", 2.7));
            StandardResistors.Add(new Component("3.0 ohm", 3.0));
            StandardResistors.Add(new Component("3.3 ohm", 3.3));
            StandardResistors.Add(new Component("3.6 ohm", 3.6));
            StandardResistors.Add(new Component("3.9 ohm", 3.9));
            StandardResistors.Add(new Component("4.3 ohm", 4.3));
            StandardResistors.Add(new Component("4.7 ohm", 4.7));
            StandardResistors.Add(new Component("5.1 ohm", 5.1));
            StandardResistors.Add(new Component("5.6 ohm", 5.6));
            StandardResistors.Add(new Component("6.2 ohm", 6.2));
            StandardResistors.Add(new Component("6.8 ohm", 6.8));
            StandardResistors.Add(new Component("7.5 ohm", 7.5));
            StandardResistors.Add(new Component("8.2 ohm", 8.2));
            StandardResistors.Add(new Component("9.1 ohm", 9.1));
            StandardResistors.Add(new Component("10 ohm", 10));
            StandardResistors.Add(new Component("11 ohm", 11));
            StandardResistors.Add(new Component("12 ohm", 12));
            StandardResistors.Add(new Component("13 ohm", 13));
            StandardResistors.Add(new Component("15 ohm", 15));
            StandardResistors.Add(new Component("16 ohm", 16));
            StandardResistors.Add(new Component("18 ohm", 18));
            StandardResistors.Add(new Component("20 ohm", 20));
            StandardResistors.Add(new Component("22 ohm", 22));
            StandardResistors.Add(new Component("24 ohm", 24));
            StandardResistors.Add(new Component("27 ohm", 27));
            StandardResistors.Add(new Component("30 ohm", 30));
            StandardResistors.Add(new Component("33 ohm", 33));
            StandardResistors.Add(new Component("36 ohm", 36));
            StandardResistors.Add(new Component("39 ohm", 39));
            StandardResistors.Add(new Component("43 ohm", 43));
            StandardResistors.Add(new Component("47 ohm", 47));
            StandardResistors.Add(new Component("51 ohm", 51));
            StandardResistors.Add(new Component("56 ohm", 56));
            StandardResistors.Add(new Component("62 ohm", 62));
            StandardResistors.Add(new Component("68 ohm", 68));
            StandardResistors.Add(new Component("75 ohm", 75));
            StandardResistors.Add(new Component("82 ohm", 82));
            StandardResistors.Add(new Component("91 ohm", 91));
            StandardResistors.Add(new Component("100 ohm", 100));
            StandardResistors.Add(new Component("110 ohm", 110));
            StandardResistors.Add(new Component("120 ohm", 120));
            StandardResistors.Add(new Component("130 ohm", 130));
            StandardResistors.Add(new Component("150 ohm", 150));
            StandardResistors.Add(new Component("160 ohm", 160));
            StandardResistors.Add(new Component("180 ohm", 180));
            StandardResistors.Add(new Component("200 ohm", 200));
            StandardResistors.Add(new Component("220 ohm", 220));
            StandardResistors.Add(new Component("240 ohm", 240));
            StandardResistors.Add(new Component("270 ohm", 270));
            StandardResistors.Add(new Component("300 ohm", 300));
            StandardResistors.Add(new Component("330 ohm", 330));
            StandardResistors.Add(new Component("360 ohm", 360));
            StandardResistors.Add(new Component("390 ohm", 390));
            StandardResistors.Add(new Component("430 ohm", 430));
            StandardResistors.Add(new Component("470 ohm", 470));
            StandardResistors.Add(new Component("510 ohm", 510));
            StandardResistors.Add(new Component("560 ohm", 560));
            StandardResistors.Add(new Component("620 ohm", 620));
            StandardResistors.Add(new Component("680 ohm", 680));
            StandardResistors.Add(new Component("750 ohm", 750));
            StandardResistors.Add(new Component("820 ohm", 820));
            StandardResistors.Add(new Component("910 ohm", 910));
            StandardResistors.Add(new Component("1.0 Kohm", 1000));
            StandardResistors.Add(new Component("1.1 Kohm", 1100));
            StandardResistors.Add(new Component("1.2 Kohm", 1200));
            StandardResistors.Add(new Component("1.3 Kohm", 1300));
            StandardResistors.Add(new Component("1.5 Kohm", 1500));
            StandardResistors.Add(new Component("1.6 Kohm", 1600));
            StandardResistors.Add(new Component("1.8 Kohm", 1800));
            StandardResistors.Add(new Component("2.0 Kohm", 2000));
            StandardResistors.Add(new Component("2.2 Kohm", 2200));
            StandardResistors.Add(new Component("2.4 Kohm", 2400));
            StandardResistors.Add(new Component("2.7 Kohm", 2700));
            StandardResistors.Add(new Component("3.0 Kohm", 3000));
            StandardResistors.Add(new Component("3.3 Kohm", 3300));
            StandardResistors.Add(new Component("3.6 Kohm", 3600));
            StandardResistors.Add(new Component("3.9 Kohm", 3900));
            StandardResistors.Add(new Component("4.3 Kohm", 4300));
            StandardResistors.Add(new Component("4.7 Kohm", 4700));
            StandardResistors.Add(new Component("5.1 Kohm", 5100));
            StandardResistors.Add(new Component("5.6 Kohm", 5600));
            StandardResistors.Add(new Component("6.2 Kohm", 6200));
            StandardResistors.Add(new Component("6.8 Kohm", 6800));
            StandardResistors.Add(new Component("7.5 Kohm", 7500));
            StandardResistors.Add(new Component("8.2 Kohm", 8200));
            StandardResistors.Add(new Component("9.1 Kohm", 9100));
            StandardResistors.Add(new Component("10 Kohm", 1e4));
            StandardResistors.Add(new Component("11 Kohm", 1.1e4));
            StandardResistors.Add(new Component("12 Kohm", 1.2e4));
            StandardResistors.Add(new Component("13 Kohm", 1.3e4));
            StandardResistors.Add(new Component("15 Kohm", 1.5e4));
            StandardResistors.Add(new Component("16 Kohm", 1.6e4));
            StandardResistors.Add(new Component("18 Kohm", 1.8e4));
            StandardResistors.Add(new Component("20 Kohm", 2e4));
            StandardResistors.Add(new Component("22 Kohm", 2.2e4));
            StandardResistors.Add(new Component("24 Kohm", 2.4e4));
            StandardResistors.Add(new Component("27 Kohm", 2.7e4));
            StandardResistors.Add(new Component("30 Kohm", 3e4));
            StandardResistors.Add(new Component("33 Kohm", 3.3e4));
            StandardResistors.Add(new Component("36 Kohm", 3.6e4));
            StandardResistors.Add(new Component("39 Kohm", 3.9e4));
            StandardResistors.Add(new Component("43 Kohm", 4.3e4));
            StandardResistors.Add(new Component("47 Kohm", 4.7e4));
            StandardResistors.Add(new Component("51 Kohm", 5.2e4));
            StandardResistors.Add(new Component("56 Kohm", 5.6e4));
            StandardResistors.Add(new Component("62 Kohm", 6.2e4));
            StandardResistors.Add(new Component("68 Kohm", 6.8e4));
            StandardResistors.Add(new Component("75 Kohm", 7.5e4));
            StandardResistors.Add(new Component("82 Kohm", 8.2e4));
            StandardResistors.Add(new Component("91 Kohm", 9.1e4));
            StandardResistors.Add(new Component("100 Kohm", 1e5));
            StandardResistors.Add(new Component("110 Kohm", 1.1e5));
            StandardResistors.Add(new Component("120 Kohm", 1.2e5));
            StandardResistors.Add(new Component("130 Kohm", 1.3e5));
            StandardResistors.Add(new Component("150 Kohm", 1.5e5));
            StandardResistors.Add(new Component("160 Kohm", 1.6e5));
            StandardResistors.Add(new Component("180 Kohm", 1.8e5));
            StandardResistors.Add(new Component("200 Kohm", 2e5));
            StandardResistors.Add(new Component("220 Kohm", 2.2e5));
            StandardResistors.Add(new Component("240 Kohm", 2.4e5));
            StandardResistors.Add(new Component("270 Kohm", 2.7e5));
            StandardResistors.Add(new Component("300 Kohm", 3e5));
            StandardResistors.Add(new Component("330 Kohm", 3.3e5));
            StandardResistors.Add(new Component("360 Kohm", 3.6e5));
            StandardResistors.Add(new Component("390 Kohm", 3.9e5));
            StandardResistors.Add(new Component("430 Kohm", 4.3e5));
            StandardResistors.Add(new Component("470 Kohm", 4.7e5));
            StandardResistors.Add(new Component("510 Kohm", 5.1e5));
            StandardResistors.Add(new Component("560 Kohm", 5.6e5));
            StandardResistors.Add(new Component("620 Kohm", 6.2e5));
            StandardResistors.Add(new Component("680 Kohm", 6.8e5));
            StandardResistors.Add(new Component("750 Kohm", 7.5e5));
            StandardResistors.Add(new Component("820 Kohm", 8.2e5));
            StandardResistors.Add(new Component("910 Kohm", 9.1e5));
            StandardResistors.Add(new Component("1.0 Mohm", 1e6));
            StandardResistors.Add(new Component("1.1 Mohm", 1.1e6));
            StandardResistors.Add(new Component("1.2 Mohm", 1.2e6));
            StandardResistors.Add(new Component("1.3 Mohm", 1.3e6));
            StandardResistors.Add(new Component("1.5 Mohm", 1.5e6));
            StandardResistors.Add(new Component("1.6 Mohm", 1.6e6));
            StandardResistors.Add(new Component("1.8 Mohm", 1.8e6));
            StandardResistors.Add(new Component("2.0 Mohm", 2e6));
            StandardResistors.Add(new Component("2.2 Mohm", 2.2e6));
            StandardResistors.Add(new Component("2.4 Mohm", 2.4e6));
            StandardResistors.Add(new Component("2.7 Mohm", 2.7e6));
            StandardResistors.Add(new Component("3.0 Mohm", 3e6));
            StandardResistors.Add(new Component("3.3 Mohm", 3.3e6));
            StandardResistors.Add(new Component("3.6 Mohm", 3.6e6));
            StandardResistors.Add(new Component("3.9 Mohm", 3.9e6));
            StandardResistors.Add(new Component("4.3 Mohm", 4.3e6));
            StandardResistors.Add(new Component("4.7 Mohm", 4.7e6));
            StandardResistors.Add(new Component("5.1 Mohm", 5.1e6));
            StandardResistors.Add(new Component("5.6 Mohm", 5.6e6));
            StandardResistors.Add(new Component("6.2 Mohm", 6.2e6));
            StandardResistors.Add(new Component("6.8 Mohm", 6.8e6));
            StandardResistors.Add(new Component("7.5 Mohm", 7.5e6));
            StandardResistors.Add(new Component("8.2 Mohm", 8.2e6));
            StandardResistors.Add(new Component("9.1 Mohm", 9.1e6));

            #endregion

            #region Add standard capacitors

            StandardCapacitors = new ObservableCollection<Component>();
            StandardCapacitors.Add(new Component("10 pf", 10e-12));
            StandardCapacitors.Add(new Component("12 pf", 12e-12));
            StandardCapacitors.Add(new Component("15 pf", 15e-12));
            StandardCapacitors.Add(new Component("18 pf", 18e-12));
            StandardCapacitors.Add(new Component("22 pf", 22e-12));
            StandardCapacitors.Add(new Component("27 pf", 27e-12));
            StandardCapacitors.Add(new Component("33 pf", 33e-12));
            StandardCapacitors.Add(new Component("39 pf", 19e-12));
            StandardCapacitors.Add(new Component("47 pf", 47e-12));
            StandardCapacitors.Add(new Component("56 pf", 56e-12));
            StandardCapacitors.Add(new Component("68 pf", 68e-12));
            StandardCapacitors.Add(new Component("82 pf", 82e-12));
            StandardCapacitors.Add(new Component("100 pf", 100e-12));
            StandardCapacitors.Add(new Component("120 pf", 120e-12));
            StandardCapacitors.Add(new Component("150 pf", 150e-12));
            StandardCapacitors.Add(new Component("180 pf", 180e-12));
            StandardCapacitors.Add(new Component("220 pf", 220e-12));
            StandardCapacitors.Add(new Component("270 pf", 270e-12));
            StandardCapacitors.Add(new Component("330 pf", 330e-12));
            StandardCapacitors.Add(new Component("390 pf", 390e-12));
            StandardCapacitors.Add(new Component("470 pf", 470e-12));
            StandardCapacitors.Add(new Component("560 pf", 560e-12));
            StandardCapacitors.Add(new Component("680 pf", 680e-12));
            StandardCapacitors.Add(new Component("820 pf", 820e-12));
            StandardCapacitors.Add(new Component("1000 pf", 1000e-12));
            StandardCapacitors.Add(new Component("1200 pf", 1200e-12));
            StandardCapacitors.Add(new Component("1500 pf", 1500e-12));
            StandardCapacitors.Add(new Component("1800 pf", 1800e-12));
            StandardCapacitors.Add(new Component("2200 pf", 2200e-12));
            StandardCapacitors.Add(new Component("2700 pf", 2700e-12));
            StandardCapacitors.Add(new Component("3300 pf", 3300e-12));
            StandardCapacitors.Add(new Component("3900 pf", 3900e-12));
            StandardCapacitors.Add(new Component("4700 pf", 4700e-12));
            StandardCapacitors.Add(new Component("5600 pf", 5600e-12));
            StandardCapacitors.Add(new Component("6800 pf", 6800e-12));
            StandardCapacitors.Add(new Component("8200 pf", 8200e-12));
            StandardCapacitors.Add(new Component("0.01 uf", 0.010e-6));
            StandardCapacitors.Add(new Component("0.012 uf", 0.012e-6));
            StandardCapacitors.Add(new Component("0.015 uf", 0.015e-6));
            StandardCapacitors.Add(new Component("0.018 uf", 0.018e-6));
            StandardCapacitors.Add(new Component("0.022 uf", 0.022e-6));
            StandardCapacitors.Add(new Component("0.027 uf", 0.027e-6));
            StandardCapacitors.Add(new Component("0.033 uf", 0.033e-6));
            StandardCapacitors.Add(new Component("0.039 uf", 0.019e-6));
            StandardCapacitors.Add(new Component("0.047 uf", 0.047e-6));
            StandardCapacitors.Add(new Component("0.056 uf", 0.056e-6));
            StandardCapacitors.Add(new Component("0.068 uf", 0.068e-6));
            StandardCapacitors.Add(new Component("0.082 uf", 0.082e-6));
            StandardCapacitors.Add(new Component("0.10 uf", 0.10e-6));
            StandardCapacitors.Add(new Component("0.12 uf", 0.12e-6));
            StandardCapacitors.Add(new Component("0.15 uf", 0.15e-6));
            StandardCapacitors.Add(new Component("0.18 uf", 0.18e-6));
            StandardCapacitors.Add(new Component("0.22 uf", 0.22e-6));
            StandardCapacitors.Add(new Component("0.27 uf", 0.27e-6));
            StandardCapacitors.Add(new Component("0.33 uf", 0.33e-6));
            StandardCapacitors.Add(new Component("0.39 uf", 0.19e-6));
            StandardCapacitors.Add(new Component("0.47 uf", 0.47e-6));
            StandardCapacitors.Add(new Component("0.56 uf", 0.56e-6));
            StandardCapacitors.Add(new Component("0.68 uf", 0.68e-6));
            StandardCapacitors.Add(new Component("0.82 uf", 0.82e-6));
            StandardCapacitors.Add(new Component("1.0 uf", 1.0e-6));
            StandardCapacitors.Add(new Component("1.2 uf", 1.2e-6));
            StandardCapacitors.Add(new Component("1.5 uf", 1.5e-6));
            StandardCapacitors.Add(new Component("1.8 uf", 1.8e-6));
            StandardCapacitors.Add(new Component("2.2 uf", 2.2e-6));
            StandardCapacitors.Add(new Component("2.7 uf", 2.7e-6));
            StandardCapacitors.Add(new Component("3.3 uf", 3.3e-6));
            StandardCapacitors.Add(new Component("3.9 uf", 1.9e-6));
            StandardCapacitors.Add(new Component("4.7 uf", 4.7e-6));
            StandardCapacitors.Add(new Component("5.6 uf", 5.6e-6));
            StandardCapacitors.Add(new Component("6.8 uf", 6.8e-6));
            StandardCapacitors.Add(new Component("8.2 uf", 8.2e-6));

            #endregion

            SingleResult = null;
            Results = null;

            ShowSingleCalcuationResults = Visibility.Collapsed;
            ShowDataGridResults = Visibility.Collapsed;

            ConnectionButtonText = BT_CONNECT;
            Connection = null;
            Endpoint = null;
            CurrentOpertion = SocketOperations.None;
            Status = "Ready";
            NetduinoIPAddress = "192.168.5.100";
            NetduinoPortNumber = 80;
            SampleRate = 10;
            FileName = "Log.txt";
            ChartVisibility = Visibility.Collapsed;
            Readings = null;
        }

        #endregion

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        #region Methods

        /// <summary>
        /// Get the value for R1.
        /// </summary>
        /// <returns>Value for resistor R1</returns>
        private double GetR1()
        {
            double result;

            result = 0;
            switch (R1ComponentValueType)
            {
                case DI_STANDARD_VALUE:
                    result = R1SelectedStandardValue.Value;
                    break;
                case DI_SPECIFIC_VALUE:
                    result = R1UserValue;
                    break;
            }
            return (result);
        }

        /// <summary>
        /// Get the value for R2.
        /// </summary>
        /// <returns>Value for resistor R2</returns>
        private double GetR2()
        {
            double result;
            result = 0;
            switch (R2ComponentValueType)
            {
                case DI_STANDARD_VALUE:
                    result = R2SelectedStandardValue.Value;
                    break;
                case DI_SPECIFIC_VALUE:
                    result = R2UserValue;
                    break;
            }
            return (result);
        }

        /// <summary>
        /// Get the value for C1.
        /// </summary>
        /// <returns>Value for capacitor C1</returns>
        private double GetC1()
        {
            double result;

            result = 0;
            switch (C1ComponentValueType)
            {
                case DI_STANDARD_VALUE:
                    result = C1SelectedStandardValue.Value;
                    break;
                case DI_SPECIFIC_VALUE:
                    result = C1UserValue;
                    break;
            }
            return (result);
        }

        /// <summary>
        /// Determine the characteristics of the NE555 given the parameters for the astable circuit.
        /// </summary>
        /// <param name="r1">Value for resistor R1</param>
        /// <param name="r2">Value for resistor R2</param>
        /// <param name="c1">Value for capacitor C1</param>
        /// <param name="f">Value for the frequency</param>
        /// <returns>Characteristics for the NE555 in the circuit.</returns>
        private ResultItem Calculate(double? r1, double? r2, double? c1, double? f)
        {
            ResultItem result;

            result = new ResultItem();
            if (r1 == null)
            {
                result.R2 = r2.Value;
                result.C1 = c1.Value;
                result.Frequency = f.Value;
                result.R2 = (1 / (f.Value * Math.Log(2) * c1.Value)) - (2 * r2.Value);
            }
            if (r2 == null)
            {
                result.R1 = r1.Value;
                result.C1 = c1.Value;
                result.Frequency = f.Value;
                result.R2 = 0.5 * ((1 / (f.Value * Math.Log(2) * c1.Value)) - r1.Value);
            }
            if (c1 == null)
            {
                result.R1 = r1.Value;
                result.R2 = r2.Value;
                result.Frequency = f.Value;
                result.C1 = 1 / (Math.Log(2) * f.Value) * (r1.Value + 2 * r2.Value);
            }
            if (f == null)
            {
                result.R1 = r1.Value;
                result.R2 = r2.Value;
                result.C1 = c1.Value;
                result.Frequency = 1 / (Math.Log(2) * c1.Value * (r1.Value + 2 * r2.Value));
            }
            result.TimeHigh = 0.693 * (r1.Value + r2.Value) * c1.Value;
            result.TimeLow = 0.693 * r2.Value * c1.Value;
            result.Period = result.TimeHigh + result.TimeLow;
            result.DutyCycle = result.R2 / (result.R1 + 2 * result.R2);
            return (result);
        }

        /// <summary>
        /// Build up a list of componets values to be processed for the DataGrid.
        /// </summary>
        /// <param name="source">Collection of components for the multiple value selection</param>
        /// <param name="standardValue">Single standard value component.</param>
        /// <param name="userValue">User selected value</param>
        /// <param name="valueType">Which type of list is required, single standard value, user value or full list</param>
        /// <param name="start">First component for a range of values.</param>
        /// <param name="end">Last component for a range of values.</param>
        /// <returns>List of components</returns>
        private List<Component> BuildComponentList(ObservableCollection<Component> source, Component standardValue, double userValue, string valueType, double start, double end)
        {
            List<Component> components;

            components = new List<Component>();
            switch (valueType)
            {
                case DI_STANDARD_VALUE:
                    components.Add(standardValue);
                    break;
                case DI_SPECIFIC_VALUE:
                    components.Add(new Component("User value", userValue));
                    break;
                case DI_RANGE_OF_VALUES:
                    foreach (Component c in StandardResistors)
                    {
                        if ((c.Value >= start) && (c.Value <= end))
                        {
                            components.Add(c);
                        }
                    }
                    break;
            }
            return (components);
        }

        /// <summary>
        /// Perform the calculations for the user.
        /// </summary>
        public bool Calculate()
        {
            bool proceed;

            proceed = false;
            if ((R1ComponentValueType == DI_RANGE_OF_VALUES) || (R2ComponentValueType == DI_RANGE_OF_VALUES) || (C1ComponentValueType == DI_RANGE_OF_VALUES))
            {
                if (SelectedMode != DI_CALCULATE_FREQUENCY)
                {
                    chwMessageBox mb;

                    mb = new chwMessageBox();
                    mb.Title = "NE555 Calculator Error";
                    mb.tbkMessage.Text = "This operation is only supported when calculating the frequency.";
                    mb.Show();
                }
                else
                {
                    List<Component> r1;
                    List<Component> r2;
                    List<Component> c1;
                    chwMessageBox mb;
                    int total;
                    int current;

                    Results = new ObservableCollection<ResultItem>();
                    r1 = BuildComponentList(StandardResistors, R1SelectedStandardValue, R1UserValue, R1ComponentValueType,
                                            R1SelectedStartValue == null ? 0 : R1SelectedStartValue.Value,
                                            R1SelectedFinalValue == null ? 0 : R1SelectedFinalValue.Value);
                    r2 = BuildComponentList(StandardResistors, R2SelectedStandardValue, R2UserValue, R2ComponentValueType,
                                            R2SelectedStartValue == null ? 0 : R2SelectedStartValue.Value,
                                            R2SelectedFinalValue == null ? 0 : R2SelectedFinalValue.Value);
                    c1 = BuildComponentList(StandardCapacitors, C1SelectedStandardValue, C1UserValue, C1ComponentValueType,
                                            C1SelectedStartValue == null ? 0 : C1SelectedStartValue.Value,
                                            C1SelectedFinalValue == null ? 0 : C1SelectedFinalValue.Value);
                    total = r1.Count * r2.Count * c1.Count;
                    proceed = true;
                    mb = new chwMessageBox();
                    mb.Title = "NE555 Calculator";
                    mb.btnOK.Visibility = Visibility.Collapsed;
                    mb.btnCancel.Visibility = Visibility.Collapsed;
                    mb.Show();
                    current = 0;
                    mb.tbkMessage.Text = string.Format("Calculation {0} of {1}", current, total);
                    foreach (Component resistor1 in r1)
                    {
                        foreach (Component resistor2 in r2)
                        {
                            foreach (Component capacitor in c1)
                            {
                                current++;
                                Results.Add(Calculate(resistor1.Value, resistor2.Value, capacitor.Value, null));
                                if ((current % 10) == 0)
                                {
                                    mb.tbkMessage.Text = string.Format("Calculation {0} of {1}", current, total);
                                }
                            }
                        }
                    }
                    mb.Close();
                    ShowDataGridResults = Visibility.Visible;
                    ShowSingleCalcuationResults = Visibility.Collapsed;
                }
            }
            else
            {
                proceed = true;
                switch (SelectedMode)
                {
                    case DI_CALCULATE_FREQUENCY:
                        SingleResult = Calculate(GetR1(), GetR2(), GetC1(), null);
                        break;
                    case DI_CALCULATE_R1:
                        SingleResult = Calculate(null, GetR2(), GetC1(), FrequencyValue);
                        break;
                    case DI_CALCULATE_R2:
                        SingleResult = Calculate(GetR1(), null, GetC1(), FrequencyValue);
                        break;
                    case DI_CALCULATE_C1:
                        SingleResult = Calculate(GetR1(), GetR2(), null, FrequencyValue);
                        break;
                }
                ShowSingleCalcuationResults = Visibility.Visible;
                ShowDataGridResults = Visibility.Collapsed;
            }
            RaisePropertyChanged("SingleResult", "Results", "ShowSingleCalcuationResults", "ShowDataGridResults");
            return (proceed);
        }

        /// <summary>
        /// Notify any subscribers that a property has changed.
        /// </summary>
        /// <param name="properties">List of name of properties which may have changed.</param>
        private void RaisePropertyChanged(params string[] properties)
        {
            if ((properties != null) && (PropertyChanged != null))
            {
                foreach (string p in properties)
                {
                    PropertyChanged(this, new PropertyChangedEventArgs(p));
                }
            }
        }

        /// <summary>
        /// Send the configuration using WebClient interface to the Netduino.
        /// </summary>
        public void SendConfiguration()
        {
            WebClient netduino;
            string url;

            netduino = new WebClient();
            netduino.DownloadStringCompleted += (s, e) =>
                {
                    if (e.Error == null)
                    {
                        Status = "Configuration accepted.";
                    }
                    else
                    {
                        Status = "WebClient: Error communicating with Netduino.";
                    }
                };
            Status = "Connecting to Netduino...";
            url = string.Format("http://{0}:{1}/Command.html?Action=C&SampleRate={2}&FileName={3}", NetduinoIPAddress, NetduinoPortNumber, SampleRate, FileName);
            netduino.DownloadStringAsync(new Uri(url));
        }

        /// <summary>
        /// Get the data file from the Netduino.
        /// </summary>
        /// <param name="fileName">Name of the file to get from the server.</param>
        public void GetDataFile(string fileName)
        {
            WebClient netduino;
            string url;

            netduino = new WebClient();
            netduino.DownloadStringCompleted += (s, e) =>
            {
                if (e.Error == null)
                {
                    string[] data;

                    data = ((string) e.Result).Split('\n');
                    Readings = new ObservableCollection<DataItem>();
                    foreach (string reading in data)
                    {
                        if (!reading.StartsWith("Tick"))
                        {
                            string[] datum;

                            datum = reading.Split(',');
                            if (datum.Length == 2)
                            {
                                DataItem item;

                                item = new DataItem();
                                item.When = long.Parse(datum[0]);
                                item.Reading = int.Parse(datum[1].Substring(0, datum[1].Length - 1));   // Remove the \r
                                Readings.Add(item);
                            }
                        }
                    }
                    RaisePropertyChanged("Readings");
                    ChartVisibility = Visibility.Visible;
                    Status = "Download complete.";
                }
                else
                {
                    Status = "WebClient: Error retrieving data file from Netduino.";
                }
            };
            Status = "Connecting to Netduino...";
            url = string.Format("http://{0}:{1}/Command.html?Action=G&FileName={2}", NetduinoIPAddress, NetduinoPortNumber, FileName);
            netduino.DownloadStringAsync(new Uri(url));
        }

        #endregion
    }
}